﻿using System.Collections.Generic;

namespace DependencyAnalysis.DataModel
{
    public class InstructionCache
    {
        private readonly Dictionary<MemberDescriptor, IList<Instruction>> _instructionsPerMember = new Dictionary<MemberDescriptor, IList<Instruction>>();

        public IList<Instruction> GetInstructions(MemberDescriptor target)
        {
            if (target.MemberType != MemberType.Method || !_instructionsPerMember.ContainsKey(target)) return new List<Instruction>();
            return _instructionsPerMember[target];
        }

        public void SetInstructions(MemberDescriptor memberDescriptor, IList<Instruction> instructions)
        {
            _instructionsPerMember[memberDescriptor] = instructions;
        }

        public static readonly ICollection<OperationCode> TypeOpCodes = new HashSet<OperationCode>
            { OperationCode.Castclass,
            OperationCode._Catch, OperationCode.Newarr, OperationCode.Box, OperationCode.Unbox, OperationCode.Unbox_Any,
            OperationCode.Isinst, OperationCode.Initobj, OperationCode.Ldtoken};

        public static readonly ICollection<OperationCode> MethodOpCodes = new HashSet<OperationCode>
            { OperationCode.Call,
            OperationCode.Callvirt, OperationCode.Calli, OperationCode.Newobj, OperationCode.Ldvirtftn, OperationCode.Ldftn,
            OperationCode.Tail_};

        public static readonly ICollection<OperationCode> FieldOpCodes = new HashSet<OperationCode>
            { OperationCode.Ldfld,
            OperationCode.Ldflda, OperationCode.Ldsfld, OperationCode.Ldsflda, OperationCode.Stfld, OperationCode.Ldftn,
            OperationCode.Stsfld, OperationCode.Ldtoken};

        public static readonly ICollection<OperationCode> LocalOpCodes = new HashSet<OperationCode>
        {
            OperationCode.Ldloc, OperationCode.Ldloc_0, OperationCode.Ldloc_1, OperationCode.Ldloc_2, OperationCode.Ldloc_3, OperationCode.Ldloc_S,OperationCode.Ldloca,OperationCode.Ldloca_S,            
            OperationCode.Stloc, OperationCode.Stloc_0, OperationCode.Stloc_1, OperationCode.Stloc_2, OperationCode.Stloc_3, OperationCode.Stloc_S           
        };

        public static readonly ICollection<OperationCode> ArgumentOpCodes = new HashSet<OperationCode>
        {
            OperationCode.Ldarg, OperationCode.Ldarg_0, OperationCode.Ldarg_1, OperationCode.Ldarg_2, OperationCode.Ldarg_3, OperationCode.Ldarg_S, OperationCode.Ldarga
        };

        public static readonly ISet<OperationCode> NestingInstructions = new HashSet<OperationCode> {OperationCode.Brfalse_S,
            OperationCode.Br_S,OperationCode.Br,OperationCode.Brfalse, OperationCode.Brtrue, OperationCode.Brtrue_S, OperationCode.Bne_Un, OperationCode.Bne_Un_S};

        public static readonly ISet<OperationCode> ArithmethicInstructions = new HashSet<OperationCode> { OperationCode.And, OperationCode.Or, OperationCode.Mul, OperationCode.Mul_Ovf,
            OperationCode.Div, OperationCode.Mul_Ovf_Un, OperationCode.Div_Un, OperationCode.Add, OperationCode.Add_Ovf, OperationCode.Add_Ovf_Un, OperationCode.Sub, OperationCode.Sub_Ovf, OperationCode.Sub_Ovf_Un,
            OperationCode.Xor};

        public static readonly ISet<OperationCode> CyclomaticInstructions = new HashSet<OperationCode> { OperationCode.And, OperationCode.Or, OperationCode.Br_S, OperationCode.Brfalse_S,
            OperationCode.Br, OperationCode.Brfalse, OperationCode.Brtrue, OperationCode.Brtrue_S, OperationCode._Catch, OperationCode.Leave, OperationCode.Leave_S, OperationCode.Bne_Un, OperationCode.Bne_Un_S};

        public static readonly ISet<OperationCode> StoreArrayElementInstructions = new HashSet<OperationCode> {
            OperationCode.Stelem, OperationCode.Stelem_I, OperationCode.Stelem_I1, OperationCode.Stelem_I2, OperationCode.Stelem_I4, OperationCode.Stelem_I8, OperationCode.Stelem_R4, OperationCode.Stelem_R8, OperationCode.Stelem_Ref};
    }
}
