﻿using System;
using System.Collections.Generic;
using DependencyAnalysis.DataModel;

namespace DependencyAnalysis.Internal
{
    public class PseudoStack
    {
        public static readonly ICollection<OperationCode> ConsumingOpCodes = new HashSet<OperationCode>
            { OperationCode.Stloc, OperationCode.Stloc_0, OperationCode.Stloc_1, OperationCode.Stloc_2, OperationCode.Stloc_3, OperationCode.Stloc_S,
             OperationCode.Stfld, 
            OperationCode.Stsfld,OperationCode.And, OperationCode.Or, OperationCode.Mul, OperationCode.Mul_Ovf,
            OperationCode.Div, OperationCode.Mul_Ovf_Un, OperationCode.Div_Un, OperationCode.Add, OperationCode.Add_Ovf, OperationCode.Add_Ovf_Un, OperationCode.Sub, OperationCode.Sub_Ovf, OperationCode.Sub_Ovf_Un,
            OperationCode.Xor};

        public static readonly ICollection<OperationCode> ProducingOpCodes = new HashSet<OperationCode>
        {OperationCode.Ldfld,OperationCode.Ldflda, OperationCode.Ldsfld, OperationCode.Ldsflda, OperationCode.Ldtoken,OperationCode.Ldftn,
            OperationCode.Ldloc, OperationCode.Ldloc_0, OperationCode.Ldloc_1, OperationCode.Ldloc_2, OperationCode.Ldloc_3, OperationCode.Ldloc_S,OperationCode.Ldloca,OperationCode.Ldloca_S, 
            OperationCode.Ldarg, OperationCode.Ldarg_0, OperationCode.Ldarg_1, OperationCode.Ldarg_2, OperationCode.Ldarg_3, OperationCode.Ldarg_S, OperationCode.Ldarga
        };

        public IList<Instruction> Instructions { get; set; }

        public Instruction TrackCallTarget(Instruction call)
        {
            var startIndex = Instructions.IndexOf(call);
            if (!InstructionCache.MethodOpCodes.Contains(call.OpCode) ||
                startIndex < 0)
            {
                throw new ArgumentException();
            }
            var calledMember = (MemberDescriptor)call.OperandNew;
            if (calledMember.IsStatic)
            {
                return call;
            }
            int reduceStack = calledMember.ParameterCount;
            int currentIndex = startIndex;
            while (reduceStack > 0 && currentIndex > -1)
            {
                currentIndex--;
                var currentInstruction = Instructions[currentIndex];
                if (ConsumingOpCodes.Contains(currentInstruction.OpCode))
                {
                    reduceStack--;
                }
                else if (ProducingOpCodes.Contains(currentInstruction.OpCode))
                {
                    reduceStack++;
                }
                else if (InstructionCache.MethodOpCodes.Contains(currentInstruction.OpCode))
                {
                    var member = (MemberDescriptor)currentInstruction.OperandNew;
                    reduceStack += member.ParameterCount;
                    if (!member.IsStatic)
                    {
                        reduceStack++;
                    }
                    if (member.ReturningType != null)
                    {
                        reduceStack--;
                    }
                }
            }
            //TODO reduce stack ==0 -> ldxxx is calltarget
            return call;
        }
    }
}
