﻿using System.Windows;
using System.Windows.Input;
using DependencyAnalysis.DataModel;
using theeg.DependencyAnalysis.CommonAddin.Models;
using theeg.GraphWpf.Arrangement;
using theeg.GraphWpf.Models;

namespace theeg.DependencyAnalysis.CommonAddin.Views
{
    public partial class ArchitectureGraph
    {
        private bool _graphInvalidated;
        private bool _initialized;

        public ICommand ChangeDescriptorCommand
        {
            get { return (ICommand)GetValue(ChangeDescriptorCommandProperty); }
            set { SetValue(ChangeDescriptorCommandProperty, value); }
        }

        public static readonly DependencyProperty ChangeDescriptorCommandProperty =
            DependencyProperty.Register("ChangeDescriptorCommand", typeof(ICommand), typeof(ArchitectureGraph), new PropertyMetadata());

        public GraphParameter GraphContent
        {
            get { return (GraphParameter)GetValue(GraphContentProperty); }
            set { SetValue(GraphContentProperty, value); }
        }

        public static readonly DependencyProperty GraphContentProperty =
            DependencyProperty.Register("GraphContent", typeof(GraphParameter), typeof(ArchitectureGraph), new PropertyMetadata());

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            if (e.Property == GraphContentProperty)
            {
                if (!_initialized)
                {
                    _graphInvalidated = true;
                }
                else
                {
                    BuildGraph();
                }
            }
            base.OnPropertyChanged(e);
        }

        private void BuildGraph()
        {
            Sketch.Nodes.Clear();
            Sketch.Transitions.Clear();
            if (GraphContent != null)
            {
                foreach (var node in GraphContent.Nodes)
                {
                    Sketch.Nodes.Add(node);
                }
                foreach (var transition in GraphContent.Transitions)
                {
                    transition.RoutingStyle = RoutingStyle.FreeHand;
                    Sketch.Transitions.Add(transition);
                }
                Sketch.Arrangement = new Layered(Sketch, 20, 80, true);
                Sketch.Arrange();
            }
            _graphInvalidated = false;
        }

        public ArchitectureGraph()
        {
            InitializeComponent();
            Sketch.Loaded += Sketch_Initialized;
        }

        private void Sketch_Initialized(object sender, System.EventArgs e)
        {
            if (_graphInvalidated)
            {
                BuildGraph();
            }
            _initialized = true;
        }

        public void ChangeDescriptor(IDescriptor descriptor)
        {
            var command = ChangeDescriptorCommand;
            if (command == null) return;
            var changeDescriptorParameter = new GraphParameter { Descriptor = descriptor };
            command.Execute(changeDescriptorParameter);
        }

        private void ClickGraphItem(object sender, RoutedEventArgs e)
        {
            var node = e.OriginalSource as Node;
            if (node != null && (node.DataObject is IDescriptor))
            {
                ChangeDescriptor(node.DataObject as IDescriptor);
            }
        }
    }
}
